#include <xparameters.h>
#include <xac97_l.h>
#include <xgpio_l.h>
#include "uart.h"
#include "game_sounds.h"
#include "ac97.h"

#define AUDIO_PLAY_TIME 5000000

void playGameSounds(void);
void ac97Reset(void);

static Xuint32 AC97_BaseAddress;
static Xuint32 AC97_StartAddress;
static Xuint32 AC97_EndAddress;

void ac97Init(Xuint32 base_address, Xuint32 buffer_start, Xuint32 buffer_end) {
  AC97_BaseAddress = base_address;
  AC97_StartAddress = buffer_start;
  AC97_EndAddress = buffer_end; 
  ac97Reset();
} // end ac97Init()


void ac97Reset() {
  Xuint32 gpio_value = XGpio_mGetDataReg(XPAR_GPIO_BASEADDR, 1);
  gpio_value &= 0xFFFFFFFE;
  XGpio_mSetDataReg(XPAR_GPIO_BASEADDR, 1, gpio_value);
  usleep(100000);
  gpio_value |= 0x00000001;
  XGpio_mSetDataReg(XPAR_GPIO_BASEADDR, 1, gpio_value);
  usleep(100000);
  XAC97_SoftReset(AC97_BaseAddress);
}


void ac97Menu() {
  Xuint8 selection;
  Xuint32 i, sample;

  clearScreen();

  do {
    print("\r\nAC97 Audio Test Menu");
    print("\r\n--------------------\r\n");
    print("1 - Passthrough Test\r\n");
    print("2 - Fifo Loopback Test\r\n");
    print("3 - Game Sounds Test\r\n");
    print("q - Quit\r\n");

    do {
      selection = uartRead();
    }while(((selection > 0) && (selection < 3)) && (selection != 'q'));

    clearScreen();

    switch(selection) {

    case '1':
      print("\r\nAC97 Audio Passthrough Test");
      print("\r\n---------------------------\r\n");
      print("Initializing audio chip...\t");
      XAC97_InitAudio(AC97_BaseAddress, AC97_ANALOG_LOOPBACK);
      print("done!\r\n");
      
      print("Playing passthrough audio...\t");
      i = AUDIO_PLAY_TIME;
      while((i-- > 0) && (selection != 'q')) {
	selection = uartScan();
      }
      selection = 0;
      print("done!\r\n");

      XAC97_SoftReset(AC97_BaseAddress);
      print("\r\nPassthrough test complete.\r\n");
      getQuitInput();
      break;

    case '2':
      print("\r\nAC97 Fifo Loopback Test");
      print("\r\n-----------------------\r\n");
      print("Initializing audio chip...\t\t");
      XAC97_InitAudio(AC97_BaseAddress, 0);
      XAC97_EnableInput(AC97_BaseAddress, AC97_LINE_INPUT);
      print("done!\r\n");

      print("Playing audio...\t\t\t");
      i = AUDIO_PLAY_TIME/8;
      while((i-- > 0) && (selection != 'q')) {
	selection = uartScan();
	sample = XAC97_ReadFifo(AC97_BaseAddress);
	XAC97_WriteFifo(AC97_BaseAddress, sample);
      }
      XAC97_SoftReset(AC97_BaseAddress);
      selection = 0;
      print("done!\r\n");

      print("\r\nFifo Loopback test complete.\r\n");
      getQuitInput();
      break;

    case '3':
      print("\r\nAC97 Game Sounds Test");
      print("\r\n---------------------\r\n");
      print("Initializing audio chip...\t");
      XAC97_InitAudio(AC97_BaseAddress, 0);
      print("done!\r\n");

      print("Playing game sounds...\t\t");
      playGameSounds();
      XAC97_SoftReset(AC97_BaseAddress);
      print("done!\r\n");

      print("\r\nGame Sounds test complete.\r\n");
      getQuitInput();
      break;

    case 'q':
      break;

    default:
      print("Invalid selection, try again!\r\n");
      break;
    }

  } while(selection != 'q');    

} // end ac97Menu()


void playGameSounds() {
  Xint32 r, i, j, strech;

  for( r = 0; r < 25; r++ ) {

    for( i = 0; i < EAT_APPLE_SOUND_LEN; i++ ) {
      j = eat_apple_sound[i];
      j = j << 6;

      for( strech = 0; strech < 4; strech++ )
	XAC97_WriteFifo(AC97_BaseAddress, j);

    }
  }

  usleep(10000);

  for( r = 0; r < 5; r++ ) {
    for( i = 0; i < DYING_SOUND_LEN; i++ ) {
      j = dying_sound[i];
      j = j << 6;

      for(strech = 0; strech < 4; strech++)
	XAC97_WriteFifo(AC97_BaseAddress, j);
    }
  }

}// end playGameSounds()

Xuint32 ac97Start(Xuint32 mode) {

  XAC97_InitAudio(AC97_BaseAddress, AC97_DIGITAL_LOOPBACK);

  if( mode == AC97_LINE_TO_LINE ) {
    XAC97_EnableInput(AC97_BaseAddress, AC97_LINE_INPUT);
	XAC97_WriteReg(AC97_BaseAddress, AC97_LineInVol, AC97_VOL_MIN);
    XAC97_WriteReg(AC97_BaseAddress, AC97_AuxOutVol, AC97_VOL_ATTN_6_0_DB  );
    XAC97_WriteReg(AC97_BaseAddress, AC97_MasterVol, AC97_VOL_MUTE);
  }
  else {
    XAC97_EnableInput(AC97_BaseAddress, AC97_MIC_INPUT);
    XAC97_WriteReg(AC97_BaseAddress, AC97_AuxOutVol, AC97_VOL_MUTE);
    XAC97_WriteReg(AC97_BaseAddress, AC97_MasterVol,AC97_VOL_ATTN_10_0_DB  );
  }

  return 1;
}


Xuint32 ac97End() {
  XAC97_SoftReset(AC97_BaseAddress);
  return 1;
}

 