#include "uart.h"
#include "ddr_sdram.h"
#include "sdram_detect.h"
#include "mem_test.h"

#define SDRAM_DATA_WIDTH 64
#define SDRAM_ADDR_WIDTH 28
#define SDRAM_WORD_COUNT 33554432 //67108864
#define SDRAM_MAX_OFFSET 0x0FFFFFFF

static Xuint32 DDR_BaseAddress;


void sdramInit(Xuint32 sdram_address) {
  DDR_BaseAddress = sdram_address;
} // end sdramInit()


void sdramTest() {
  Xuint32 data_width = SDRAM_DATA_WIDTH;
  Xuint32 addr_width = SDRAM_ADDR_WIDTH;
  Xuint32 word_count = SDRAM_WORD_COUNT;
  Xuint32 max_offset = SDRAM_MAX_OFFSET;
  Xuint32 base_address, i;
  Xuint32 rank_count;

  clearScreen();

  print("\r\nSDRAM Module Detection");
  print("\r\n----------------------\r\n");
  print("Checking available DIMM module...\t");
  rank_count = sdramDetect();
  if( rank_count <= 0 ) { 
    getQuitInput();
    return;
  }
  print("done!\r\n");
  xil_printf("Ranks detected: %d\r\n", rank_count);

  for( i = 0; i < rank_count; i++ ) {
    base_address = DDR_BaseAddress + (i * 0x10000000);
    xil_printf("\r\nDDR SDRAM Test: Rank %d", i);
    print("\r\n----------------------\r\n");
    runDataTestL(base_address, data_width);
    runAddrTest(base_address, addr_width, max_offset);
    runDeviceTestL(base_address, word_count);
  }

  print("\r\nMemory test complete\r\n");
  getQuitInput();

} // end sdramTest()


Xuint8 sdramQuiet() {
  Xuint32 data_width = SDRAM_DATA_WIDTH;
  Xuint32 addr_width = SDRAM_ADDR_WIDTH;
  Xuint32 max_offset = SDRAM_MAX_OFFSET;
  Xuint32 rank_count;
  Xuint32 base_address, i;

  rank_count = sdramDetectX(0);

  if( rank_count == 0 ) {
    print("error.supported-dimm-not-found\n");
    return 0;
  }

  for( i = 0; i < rank_count; i++ ) {
    base_address = DDR_BaseAddress + (i * 0x10000000);
    if(!runDataTestLX(base_address, data_width, 0)) 
      return 0;
    if(!runAddrTestX(base_address, addr_width, max_offset, 0))
      return 0;
  }

  return 1;
}
