#include <xparameters.h>
#include <xuartlite_l.h>
#include "uart.h" 
#include "mgt_sata.h"
#include "web.h"
#include "ac97.h"
#include "sysace.h"
#include "sdram_detect.h"
#include "ddr_sdram.h"
#include "mac_phy_test.h"

#define CPU_FREQUENCY 100

#define AUDIO_START_ADDR 0x00000000
#define AUDIO_END_ADDR   0x003FFFFF
#define PB_ENTER_AND_MASK    0x00000010
#define PB_RIGHT_AND_MASK    0x00000020
#define PB_LEFT_AND_MASK     0x00000040
#define PB_DOWN_AND_MASK     0x00000080
#define PB_UP_AND_MASK       0x00000100
#define MFGMODE 0
#define VERBOSE 1

void hwInit(void);
int mfgtest_menu(void);

int main() {
  unsigned char x, s;


  hwInit();
  clearScreen();
  // turn off the mgts
  force_powerdown(0);
 

  do {

    print("\r\nXUP-V2Pro BuiltIn Self Test Main Menu Rev. 1.5 June. 2005");
    print("\r\n----------------------------------------------------------\r\n");
    print("1 - Test SATA port with Aurora loopback.\r\n");
    print("2 - Test Ethernet with WEB example.\r\n");
    print("3 - Test AC97 audio codec.\r\n");
    print("4 - Test System ACE.\r\n");
    print("5 - Test DDR SDRAM.\r\n");
	print("6 - Test Expansion connectors.\r\n");
    print("q - Quit\r\n");
	print("\0\0\0\n");  // send to for MFG test to know when to start

    do {
      s = uartRead();
    } while((s < '1' || s > '6') && s != 'x' && s != 'q');

    switch(s) {
    case '1':
      mgtMenu();
      break;
    case '2':
      web();
      break;
    case '3':
      ac97Menu();
      break;
    case '4':
      sysaceTest(VERBOSE);
      break;
    case '5':
      sdramTest();
      break;
    case '6':
      expansion_hw_bist();
      break;
	case 'x':
	  mfgtest_menu();
      break;
    }

  } while( s != 'q' );

  print("\r\nGoodbye.\r\n");

  return 0;

} // end main()


void hwInit() {

  uartInit(XPAR_UART_BASEADDR);
  periphInit(XPAR_GPIO_BASEADDR);  
  i2cInit(XPAR_I2C_BASEADDR);
  sdramInit(XPAR_DDR_SDRAM_MEM0_BASEADDR);
  webInit(XPAR_EMAC_BASEADDR);
  ac97Init(XPAR_AC97_BASEADDR, AUDIO_START_ADDR, AUDIO_END_ADDR);
  sysaceInit(XPAR_SYSACE_BASEADDR);
  mgtInit(XPAR_AT_BASEADDR, CPU_FREQUENCY);

} // end hwInit()

int mfgtest_menu(void)
{
 unsigned char UP_pushed = 0;
 unsigned char DN_pushed = 0;
 unsigned char LFT_pushed = 0;
 unsigned char RGT_pushed = 0;
 unsigned char ENT_pushed = 0;
 volatile unsigned int switches;
 volatile int *psw_data;
 unsigned char s = 0;

 psw_data = (int *) XPAR_GPIO_BASEADDR + 2;

while(1)
{
   switches = *(psw_data);
   //putnum(switches);
   if(s == 'y'){
   	XGpio_mSetDataReg(XPAR_GPIO_BASEADDR, 1,1);
   return;
   }
// detect pushbutton presses
     if( !(PB_ENTER_AND_MASK  & switches) && ! ENT_pushed )  // detect pushbutton press on ENTER
     { 
     print("p.ENTER\n");
	 ENT_pushed = 1;
	 }
   if( !(PB_RIGHT_AND_MASK  & switches) && ! RGT_pushed )   // detect pushbutton press on RIGHT
     { 
     print("p.RIGHT\n");
	 RGT_pushed = 1;
	 } 
   if( !(PB_LEFT_AND_MASK  & switches) && ! LFT_pushed )    // detect pushbutton press on LEFT
     { 
     print("p.LEFT\n");
	 LFT_pushed = 1;
	 }       
   if( !(PB_DOWN_AND_MASK  & switches) && ! DN_pushed )    // detect pushbutton press on DOWN
     { 
     print("p.DOWN\n");
	 DN_pushed = 1;
	 }
   if( !(PB_UP_AND_MASK  & switches) && ! UP_pushed)       // detect pushbutton press on UP
     { 
     print("p.UP\n");
	 UP_pushed = 1;
	 }
// detect pushbutton release
     if( (PB_ENTER_AND_MASK  & switches) &&  ENT_pushed )   // detect pushbutton press on ENTER
     { 
     print("q.ENTER\n");
	 ENT_pushed = 0;
	 }
   if( (PB_RIGHT_AND_MASK  & switches) &&  RGT_pushed )     // detect pushbutton press on RIGHT
     { 
     print("q.RIGHT\n");
	 RGT_pushed = 0;
	 } 
   if( (PB_LEFT_AND_MASK  & switches) &&  LFT_pushed )      // detect pushbutton press on LEFT
     { 
     print("q.LEFT\n");
	 LFT_pushed = 0;
	 }       
   if( (PB_DOWN_AND_MASK  & switches) &&  DN_pushed )       // detect pushbutton press on DOWN
     { 
     print("q.DOWN\n");
	 DN_pushed = 0;
	 }
   if( (PB_UP_AND_MASK  & switches) &&  UP_pushed)          // detect pushbutton press on UP
     { 
     print("q.UP\n");
	 UP_pushed = 0;
	 }
	  s = uartScan();
	  if(((s < 'a' || s > 'r')  && s != 'x'  && s != 'y' && s != 0))
	  {
	  print("z\n");
	  }
    switch(s) {
    case 'a':
      loop_0to1_test(MFGMODE, 1);  //sata0to1
      break;            
    case 'b':
      loop_1to2_test(MFGMODE, 1);  //sata2to1
      break;
    case 'c':
      print("c.");
      if(sdramQuiet())
	  print("pass\n");  //dimm
	  else
	  print("c.fail\n");
      break;  
    case 'd':            
	  userdip1();         //userdip1        
      break;
    case 'e':            
	  userdip2();         //userdip2
      break;
    case 'f':
      print("f.");
      if(ac97Start(AC97_LINE_TO_LINE))
	print("pass\n");  //audiotoaudiostart
      break;
    case 'g':
      print("g.");
      if(ac97End())
	print("pass\n");  //audiotoaudioend
      break;
    case 'h':
      print("h.");
      if(ac97Start(AC97_MIC_TO_AMP))
	print("pass\n");  //mictoampstart   
      break;
    case 'i':
      print("i.");
      if(ac97End())
	print("pass\n");  //micotampend             
      break;	  
	case 'j':
      conntest1();        //conntest1              
	  break;	  
	case 'k':
      sysaceTest(MFGMODE);  //sysace          
	  break;	  
	case 'l':
      kbdps2();           //kbdps2
	  break;	  
	case 'm':
      mouseps2();         //mouseps2                
	  break;	  
    case 'n' :            //Ethernet
      if (emac_mfg_test())
        print("n.fail\n");
      else
        print("n.pass\n");
      break;	  	  
	case 'o':
      print_serial_num(); //sn               
	  break;
	  /*
	case 'p':
      print("p.pass\n");  //userbtndwn              
	  break;	      	  	   
	case 'q':
      print("q.pass\n");  //userbtnup                            
	  break;
	  */
	case 'r':
	  conntest2();       //conntest2  
	  break;
    }
 }

}// end mfgtest_menu



