#include "global_types.h"
#include "uart.h"
#include "mgt_sata.h"

#define WORD_BYTE_LENGTH 2 

static LWORD FRAME_SIZE1 = 0x040;  //set 64 frame size initially
static LWORD FRAME_SIZE2 = 0x040;  //set 64 frame size initially
static BOOL CLOCK_SEL[2] = {FALSE, FALSE}; //use the board oscillators
static BOOL input_ufc[2] = {FALSE, FALSE};
static BYTE  NFC_NB[2] = {0x00,0x00};
static LWORD stop_cycle[2];  // 64-bit word
static LWORD rx_words0_nodiv; // 32-bit word
static LWORD rx_words_prv0; // 32-bit word
static LWORD delta0;
static LWORD rate0;
static LWORD rx_words1_nodiv; // 32-bit word
static LWORD rx_words_prv1; // 32-bit word
static LWORD delta1;
static LWORD rate1;
static WORD dip_switch1 = 0x0000;
static WORD push_button = 0x0000;
static WORD nfc_number  = 0x0000;
static WORD dip_switch2 = 0x0000;
static WORD dip_switch3 = 0x0000;
static WORD dip_switch4 = 0x0000;
static BOOL RandFrmSzEn[2] = {FALSE,FALSE};
static BOOL PowerDown[2]   = {FALSE,FALSE};
static BOOL Pattern[2]     = {FALSE,FALSE};
static BOOL sLoopback[2] = {FALSE,FALSE};
static BOOL pLoopback[2] = {FALSE,FALSE};
static BOOL IntLoopback[2] = {FALSE,FALSE};
static BOOL nfc_enable[2] = {FALSE,FALSE};
static BOOL clk_detected = FALSE;
static BOOL config_code[3] = {FALSE,FALSE,FALSE};
static LWORD bert_config_status;
static LWORD UFC_TX_MSG[2][4];
static LWORD UFC_RX_MSG[2][4];

static char dash40[] = {"----------------------------------------"};
static char dash14[] = {"--------------"};

void aurora_test(int print_mode, int end_frame_num);
LWORD read_dword_data(int sel);
LWORD read_lword_data(WORD offset);
void write_lsb_word_data(WORD d, WORD offset);
void write_msb_word_data(WORD d, WORD offset);
void aurora_status_monitor(int print_mode, int end_frame_num);
void toggle_randfrmszen(int sel);
void toggle_powerdown(int sel);
void toggle_sloopback(int sel);
void toggle_ploopback(int sel);
void toggle_intloopback(int sel);
void toggle_nfc_enable(int sel);
int  reset_sequence(int print_mode);
void master_reset_pulse();
void aurora_reset();
void change_frame_size();
void change_clock_source();
void inc_nfc(int sel);
void dec_nfc(int sel);
BOOL  get_bool(LWORD v);
LWORD get_cycles();
void set_frame_size();
void set_clock_source();
void toggle_pattern(int sel);
char int2char(LWORD a);
void print_char_array(DLWORD a);
void print_lword(LWORD a);
void print_ufc_msg(int a);
LWORD pow(LWORD x, LWORD y);
void write_ufc(int sel);
void read_ufc(int sel);
void trigger_ufc(int sel);
void force_powerdown(int sel);
void loop_0to1_test(int print_mode, int end_frame_num);
void loop_1to2_test(int print_mode, int end_frame_num);

static Xuint32 CPU_Frequency;
static Xuint32 BERT_BaseAddress;


void mgtInit(Xuint32 bert_address, Xuint32 cpu_freq) {
  BERT_BaseAddress = bert_address;
  CPU_Frequency = cpu_freq;
}


void mgtMenu() {
  char s;
  int print_mode = 1;
  int end_frame_num = 1000;
  s = '0';


  clearScreen();

  do {
    print("\r\nMGT SATA Test Menu");
    print("\r\n------------------\r\n");
    print("Verify the SATA cable is connected between:\r\n");
    print("  SATA0 Host and SATA1 Target\r\n");
    print("              or\r\n"); 
    print("  SATA2 Host and SATA1 Target\r\n");
    //print(" NOTE: VGA Display Will Turn Off While Testing SATA Ports!!");
    print("\r\n\r\n");
    print("1 - Run loop test on SATA0 Host and SATA1 Target.\r\n");
    print("2 - Run loop test on SATA2 Host and SATA1 Target.\r\n");
    print("q - Quit.\r\n");

    do {
      s = uartRead();
    } while((s < '1' || s >'5') && s != 'q');
	
    if (s == '1') {
	master_reset_pulse();
    usleep(1000000);// wait a while
    loop_0to1_test(print_mode, end_frame_num);
    } 
    else if (s == '2') {
	master_reset_pulse();
    usleep(1000000);  // wait a while
    loop_1to2_test(print_mode, end_frame_num);
    }
 
  }while (s != 'q');


  clearScreen();

}


void aurora_test(int print_mode, int end_frame_num) {
  char s;


  // disable the cache because the bert doesn't need them
  XCache_DisableICache();
  XCache_DisableDCache();

  s='0';
  if(print_mode)
  {
    clearScreen();

    print("\r\nAurora Test System Version 2.1");
    print("\r\n------------------------------");
   }
    uartScan();


   aurora_status_monitor(print_mode, end_frame_num);
  if(print_mode){
  clearScreen();
  }

}


void aurora_status_monitor(int print_mode, int end_frame_num) {
 LWORD drop_frames0=0;
 LWORD drop_frames1=0;
 LWORD drop_words0=0;
 LWORD drop_words1=0;

 LWORD tx_words0=0;
 LWORD tx_words1=0;
 LWORD rx_words0=0;
 LWORD rx_words1=0;
 LWORD tx_frames0=0;
 LWORD tx_frames1=0;
 LWORD rx_frames0=0;
 LWORD rx_frames1=0;

 LWORD tx_words0_high=0;
 LWORD tx_words1_high=0;
 LWORD rx_words0_high=0;
 LWORD rx_words1_high=0;
 LWORD tx_frames0_high=0;
 LWORD tx_frames1_high=0;
 LWORD rx_frames0_high=0;
 LWORD rx_frames1_high=0;

 DLWORD tx_words0_full=0;
 DLWORD tx_words1_full=0;
 DLWORD rx_words0_full=0;
 DLWORD rx_words1_full=0;
 DLWORD tx_frames0_full=0;
 DLWORD tx_frames1_full=0;
 DLWORD rx_frames0_full=0;
 DLWORD rx_frames1_full=0;

 LWORD tx_words0_int=0;
 LWORD tx_words1_int=0;
 LWORD rx_words0_int=0;
 LWORD rx_words1_int=0;
 LWORD tx_frames0_int=0;
 LWORD tx_frames1_int=0;
 LWORD rx_frames0_int=0;
 LWORD rx_frames1_int=0;

 LWORD tx_words0_prv=0;
 LWORD tx_words1_prv=0;
 LWORD rx_words0_prv=0;
 LWORD rx_words1_prv=0;
 LWORD tx_frames0_prv=0;
 LWORD tx_frames1_prv=0;
 LWORD rx_frames0_prv=0;
 LWORD rx_frames1_prv=0;

 LWORD hard_err_cnt0=0;
 LWORD soft_err_cnt0=0;  
 LWORD framing_err_cnt0=0;
 LWORD cc_cnt0=0;
 LWORD hard_err_cnt1=0;
 LWORD soft_err_cnt1=0;  
 LWORD framing_err_cnt1=0;
 LWORD cc_cnt1=0;

 LWORD delta_cycles=0;
 LWORD bert_status;

 BOOL mgt_intloopbk[2];
 BOOL mgt_linked[2];
 BOOL mgt_powrdn[2];
 BOOL mgt_loopbk[4];
	
 LWORD temp_lword;

 int i, l, m;
 int e;
 char s;
 char ufc_char;
 int num_chars;
 char str[2];
 int failed_link = 0;


 e = 0;
 str[1] = '\0';

 //change_frame_size();
  failed_link = reset_sequence(print_mode); // reset the bert, clear the frame counters
  while(uartScan() != 'q' && e == 0) {
    if(print_mode){
	    clearScreen();
		}
    bert_status = read_dword_data(17); // sample on the bert status
    bert_status = read_dword_data(17); // sample again on the bert status
    mgt_intloopbk[0]    = get_bool(bert_status & 0x00000004);
    mgt_intloopbk[1]    = get_bool(bert_status & 0x00000400);
    mgt_linked[0] = get_bool(bert_status & 0x00000008);
    mgt_linked[1] = get_bool(bert_status & 0x00000800);
    mgt_powrdn[0] = get_bool(bert_status & 0x00000040);
    mgt_powrdn[1] = get_bool(bert_status & 0x00004000);
    mgt_loopbk[0] = get_bool(bert_status & 0x00000010);
    mgt_loopbk[1] = get_bool(bert_status & 0x00000020);
    mgt_loopbk[2] = get_bool(bert_status & 0x00001000);
    mgt_loopbk[3] = get_bool(bert_status & 0x00002000);

    delta_cycles     = get_cycles(); // stop timer, get processing time, unit: ppc cycles
    rx_words_prv0    = rx_words0_nodiv;
    rx_words_prv1    = rx_words1_nodiv;

    tx_frames0_prv   = tx_frames0_int;
    tx_words0_prv    = tx_words0_int;
    rx_frames0_prv   = rx_frames0_int;
    rx_words0_prv    = rx_words0_int;;

    tx_frames0_int   = read_dword_data(2);    
    tx_words0_int    = read_dword_data(6);
    rx_frames0_int   = read_dword_data(4);
    rx_words0_int    = read_dword_data(8);  
    rx_words0_nodiv  = read_dword_data(8);
    drop_frames0     = read_dword_data(10);
    drop_words0      = read_dword_data(12);
    hard_err_cnt0    = read_dword_data(13) & 0x0000FFFF;
    soft_err_cnt0    = read_dword_data(14) & 0x0000FFFF;
    framing_err_cnt0 = read_dword_data(15) & 0x0000FFFF;
    cc_cnt0          = read_dword_data(16) & 0x0000FFFF;

    tx_frames1_prv   = tx_frames1_int;
    tx_words1_prv    = tx_words1_int;
    rx_frames1_prv   = rx_frames1_int;
    rx_words1_prv    = rx_words1_int;

    tx_frames1_int   = read_dword_data(1);
    tx_words1_int    = read_dword_data(5);
    rx_frames1_int   = read_dword_data(3);
    rx_words1_int    = read_dword_data(7);
    rx_words1_nodiv  = read_dword_data(7);
    drop_frames1     = read_dword_data(9);
    drop_words1      = read_dword_data(11);
    hard_err_cnt1    = read_dword_data(13) & 0xFFFF0000;
    soft_err_cnt1    = read_dword_data(14) & 0xFFFF0000;
    framing_err_cnt1 = read_dword_data(15) & 0xFFFF0000;
    cc_cnt1          = read_dword_data(16) & 0xFFFF0000;

    //////////////////////Aurora0
    if (tx_frames0_int >= tx_frames0_prv)
      tx_frames0_int = tx_frames0_int - tx_frames0_prv + tx_frames0;
    else //overflow in hardware
      tx_frames0_int = (0xFFFFFFFFFFFFFFFF - tx_frames0_prv) + tx_frames0_int  + tx_frames0;

    if (tx_frames0_int < tx_frames0) //overflow the total count in software
      tx_frames0_high++;

    tx_frames0 = tx_frames0_int;
    tx_frames0_full = tx_frames0_high;
    tx_frames0_full<<=32;
    tx_frames0_full+=tx_frames0_int;

    if (tx_words0_int >= tx_words0_prv)
      tx_words0_int = tx_words0_int - tx_words0_prv + tx_words0;
    else //overflow in hardware
      tx_words0_int = (0xFFFFFFFFFFFFFFFF - tx_words0_prv) + tx_words0_int  + tx_words0;

    if (tx_words0_int < tx_words0) //overflow the total count in software
      tx_words0_high++;

    tx_words0 = tx_words0_int;
    tx_words0_full = tx_words0_high;
    tx_words0_full<<=32;
    tx_words0_full+=tx_words0_int;

    if (rx_frames0_int >= rx_frames0_prv)
      rx_frames0_int = rx_frames0_int - rx_frames0_prv + rx_frames0;
    else //overflow in hardware
      rx_frames0_int = (0xFFFFFFFFFFFFFFFF - rx_frames0_prv) + rx_frames0_int  + rx_frames0;

    if (rx_frames0_int < rx_frames0) //overflow the total count in software
      rx_frames0_high++;

    rx_frames0 = rx_frames0_int;
    rx_frames0_full = rx_frames0_high;
    rx_frames0_full<<=32;
    rx_frames0_full+=rx_frames0_int;

    if (rx_words0_int >= rx_words0_prv)
      rx_words0_int = rx_words0_int - rx_words0_prv + rx_words0;
    else //overflow in hardware
      rx_words0_int = (0xFFFFFFFFFFFFFFFF - rx_words0_prv) + rx_words0_int  + rx_words0;

    if (rx_words0_int < rx_words0) //overflow the total count in software
      rx_words0_high++;

    rx_words0 = rx_words0_int;
    rx_words0_full = rx_words0_high;
    rx_words0_full<<=32;
    rx_words0_full+=rx_words0_int;
    
    //////////////////////Aurora1
    if (tx_frames1_int >= tx_frames1_prv)
      tx_frames1_int = tx_frames1_int - tx_frames1_prv + tx_frames1;
    else //overflow in hardware
      tx_frames1_int = (0xFFFFFFFFFFFFFFFF - tx_frames1_prv) + tx_frames1_int  + tx_frames1;

    if (tx_frames1_int < tx_frames1) //overflow the total count in software
      tx_frames1_high++;

    tx_frames1 = tx_frames1_int;
    tx_frames1_full = tx_frames1_high;
    tx_frames1_full<<=32;
    tx_frames1_full+=tx_frames1_int;

    if (tx_words1_int >= tx_words1_prv)
      tx_words1_int = tx_words1_int - tx_words1_prv + tx_words1;
    else //overflow in hardware
      tx_words1_int = (0xFFFFFFFFFFFFFFFF - tx_words1_prv) + tx_words1_int  + tx_words1;

    if (tx_words1_int < tx_words1) //overflow the total count in software
      tx_words1_high++;

    tx_words1 = tx_words1_int;
    tx_words1_full = tx_words1_high;
    tx_words1_full<<=32;
    tx_words1_full+=tx_words1_int;

    if (rx_frames1_int >= rx_frames1_prv)
      rx_frames1_int = rx_frames1_int - rx_frames1_prv + rx_frames1;
    else //overflow in hardware
      rx_frames1_int = (0xFFFFFFFFFFFFFFFF - rx_frames1_prv) + rx_frames1_int  + rx_frames1;

    if (rx_frames1_int < rx_frames1) //overflow the total count in software
      rx_frames1_high++;

    rx_frames1 = rx_frames1_int;
    rx_frames1_full = rx_frames1_high;
    rx_frames1_full<<=32;
    rx_frames1_full+=rx_frames1_int;

    if (rx_words1_int >= rx_words1_prv)
      rx_words1_int = rx_words1_int - rx_words1_prv + rx_words1;
    else //overflow in hardware
      rx_words1_int = (0xFFFFFFFFFFFFFFFF - rx_words1_prv) + rx_words1_int  + rx_words1;

    if (rx_words1_int < rx_words1) //overflow the total count in software
      rx_words1_high++;

    rx_words1 = rx_words1_int;
    rx_words1_full = rx_words1_high;
    rx_words1_full<<=32;
    rx_words1_full+=rx_words1_int;

    XTime_SetTime(0); // clear timer
    delta0 = rx_words0_nodiv - rx_words_prv0;
    delta1 = rx_words1_nodiv - rx_words_prv1;

    // clock detection
    if ((mgt_linked[0] && (delta0 >0) ) || (mgt_linked[1] && (delta1 >0)))
      {
	clk_detected = TRUE;
      }

    // modified formula to prevent overflow, unit: Mbps
    // The formula ensures no overflow on the 32-bits calculation
    if ((delta0 == 0 && !clk_detected) || rx_words_prv0 == 0)    { rate0 = 0; }
    else
      rate0 = (delta0 * WORD_BYTE_LENGTH /125000 *CPU_Frequency) / (delta_cycles / 1000000);

    if ((delta1 == 0 && !clk_detected) || rx_words_prv1 == 0)    {rate1 = 0; }
    else
      rate1 = (delta1 * WORD_BYTE_LENGTH /125000 *CPU_Frequency) / (delta_cycles / 1000000);
    
    
//-------------------------------------------------------------------------------
// Active MGT Status
//-------------------------------------------------------------------------------
  if(print_mode){
    xil_printf("+%s",(char *) dash40);  
    xil_printf("%s\r\n", (char *) dash14);
	if (CLOCK_SEL[0]){
    	xil_printf("| ML-XUP LINK    |     SATA0      | SATA1\r\n");
		} 
  	else{
     	xil_printf("| ML-XUP LINK    |     SATA2      | SATA1\r\n");
		} 
    xil_printf("|--------------%s\r\n",(char *)dash40);
    xil_printf("| Aurora Status  |    Aurora0      Aurora1\r\n");
    xil_printf("|--------------%s\r\n",(char *)dash40);
    xil_printf("| TX Frames (64k)| ");
    print_char_array((tx_frames0_full>>16));
    xil_printf(" ");
    print_char_array((tx_frames1_full>>16));
    xil_printf("\r\n");
    xil_printf("| TX Words  (64k)| ");
    print_char_array((tx_words0_full>>16));
    xil_printf(" ");
    print_char_array((tx_words1_full>>16));
    xil_printf("\r\n");
    xil_printf("| TX CCs    (64k)| ");
    print_char_array(cc_cnt0);
    xil_printf(" ");
    print_char_array(cc_cnt1>>16);
    xil_printf("\r\n");

    xil_printf("|--------------%s\r\n",(char *)dash40);
    xil_printf("| RX Rate (Mbps) | ");
    if (rate0 > 0) 
      print_char_array(rate0);
    else
      xil_printf("        n/a");
    xil_printf(" ");
    if (rate1 > 0)
      print_char_array(rate1);
    else
      xil_printf("        n/a");
    xil_printf("\r\n");

    xil_printf("| RX Frames (64k)| ");
    print_char_array((rx_frames0_full>>16));
    xil_printf(" ");
    print_char_array((rx_frames1_full>>16));
    xil_printf("\r\n");
    xil_printf("| RX Words  (64k)| ");
    print_char_array((rx_words0_full>>16));
    xil_printf(" ");
    print_char_array((rx_words1_full>>16));
    xil_printf("\r\n");

    xil_printf("|--------------%s\r\n",(char *)dash40);
    xil_printf("| Dropped Frames | ");
    print_char_array(drop_frames0);
    xil_printf(" ");
    print_char_array(drop_frames1);
    xil_printf("\r\n");
    xil_printf("| Dropped Words  | ");
    print_char_array(drop_words0);
    xil_printf(" ");
    print_char_array(drop_words1);
    xil_printf("\r\n");

    xil_printf("| Hard Errors    | ");
    print_char_array(hard_err_cnt0);
    xil_printf(" ");
    print_char_array(hard_err_cnt1>>16);
    xil_printf("\r\n");
    xil_printf("| Soft Errors    | ");
    print_char_array(soft_err_cnt0);
    xil_printf(" ");
    print_char_array(soft_err_cnt1>>16);
    xil_printf("\r\n"); 
    xil_printf("| Framing Errors | ");
    print_char_array(framing_err_cnt0);
    xil_printf(" ");
    print_char_array(framing_err_cnt1>>16);
    xil_printf("\r\n");

    xil_printf("|--------------%s\r\n",(char *)dash40);
    xil_printf("| Link           |           %d", mgt_linked[0]);
    xil_printf("           %d\r\n", mgt_linked[1]);
    xil_printf("| Pattern        | ");
    if (Pattern[0])
      xil_printf("     random");
    else
      xil_printf(" sequential");
    xil_printf(" ");
    if (Pattern[1])
      xil_printf("     random");
    else
      xil_printf(" sequential");
    xil_printf("\r\n"); 
    xil_printf("| Frame Size     | ");
    if (RandFrmSzEn[0])
      xil_printf("     random");
    else if (FRAME_SIZE1==0)
      xil_printf("   infinite");
    else
      print_char_array(FRAME_SIZE1);  
    xil_printf(" ");
    if (RandFrmSzEn[1])
      xil_printf("     random"); 
   else if (FRAME_SIZE2==0)
      xil_printf("   infinite");
    else 
      print_char_array(FRAME_SIZE2);
    xil_printf("\r\n");   
    xil_printf("| LoopBack       |          %d", mgt_loopbk[1]);
    xil_printf("%d", mgt_loopbk[0]);
    xil_printf("          %d", mgt_loopbk[3]);
    xil_printf("%d\r\n", mgt_loopbk[2]);
    xil_printf("| PowerDown      |           %d", mgt_powrdn[0]);
    xil_printf("           %d\r\n", mgt_powrdn[1]);
/*
    xil_printf("| Internal Echo  |           %d", mgt_intloopbk[0]);
    xil_printf("           %d\r\n", mgt_intloopbk[1]);  
    xil_printf("| NFC Number     | ");
    if (nfc_enable[0])
      if (NFC_NB[0]==15)
	xil_printf("       XOFF");
      else if (NFC_NB[0]==0)
	xil_printf("        XON");
      else
	print_char_array(NFC_NB[0]);
    else 
      xil_printf("   disabled");

    xil_printf(" ");
    if (nfc_enable[1])
      if (NFC_NB[1]==15)
	xil_printf("       XOFF");
      else if (NFC_NB[1]==0)
	xil_printf("        XON");
      else
	print_char_array(NFC_NB[1]);
    else
      xil_printf("   disabled");
    xil_printf("\r\n");

    xil_printf("|--------------%s\r\n", (char*)dash40);
    print_ufc_msg(0);
    xil_printf("|--------------%s\r\n",(char *)dash40);
    print_ufc_msg(1);
*/
    
    xil_printf("|--------------%s\r\n",(char *)dash40);
    xil_printf("| [1] Toggle Aurora0 Pattern [7] Toggle Aurora1 Pattern\r\n");
    xil_printf("| [2] Toggle Aurora0 RndFmSz [8] Toggle Aurora1 RndFmSz\r\n");
    xil_printf("| [3] Toggle Aurora0 P. LpBk [9] Toggle Aurora1 P. LpBk\r\n");
    xil_printf("| [4] Toggle Aurora0 S. LpBk [0] Toggle Aurora1 S. LpBk\r\n");
    xil_printf("| [5] Toggle Aurora0 PwrDwn  [-] Toggle Aurora1 PwrDwn\r\n");
/* 
    xil_printf("| [6] Toggle Aurora0 Echo    [=] Toggle Aurora1 Echo\r\n");
    xil_printf("| [q] Toggle Aurora0 NFC on  [w] Toggle Aurora1 NFC on\r\n");
    xil_printf("| [e] Send UFC from Aurora0  [r] Send UFC from Aurora1\r\n");
    xil_printf("| [a/z] inc/dec Aurora0 NFC  [s/x] inc/dec Aurora1 NFC\r\n"); 
*/
    xil_printf("| [space] Reset Both MGTs    [q] Quit\r\n");
    xil_printf("+%s",(char *) dash40);  xil_printf("%s\r\n", (char *) dash14);
 }// end of if print mode
    i = 0;
    while (i<256000 && e == 0)
      {
        s = uartScan();
        if (s == 'q' || ((tx_frames0_full>>16) > end_frame_num) ||  ((tx_frames1_full>>16) > end_frame_num) || failed_link == 1)
          { e = 1;} 
		else if (s == 'z') 
			{
			break;
			}
		else if (s == '1')
          {
            toggle_pattern(0);
            break;
          }
        else if (s == '2')
          {
            toggle_randfrmszen(0);	
	    aurora_reset();
            break;
          }
        else if (s == '3')
          {
            toggle_ploopback(0);
	    aurora_reset();
            break;
          }
        else if (s == '4')
          {
            toggle_sloopback(0);
	    aurora_reset();
            break;
          } 
        else if (s == '5')
          {
            toggle_powerdown(0);
            break;
          }
		else if (s == '6')
          {
            toggle_intloopback(0);
	    aurora_reset();
            break;
          }	
		else if (s == 'q')
          {
            toggle_nfc_enable(0);
            break;
          } 
        else if (s == 'a')
          {
            inc_nfc(0);
            break;
          } 
        else if (s == 'z')
          {
            dec_nfc(0);
            break;
          }
		else if (s == '7')
          {
            toggle_pattern(1);
            break;
          }
        else if (s == '8')
          {
            toggle_randfrmszen(1);
	    aurora_reset();
            break;
          }
        else if (s == '9')
          {
            toggle_ploopback(1);
	    aurora_reset();
            break;
          }
        else if (s == '0')
          {
            toggle_sloopback(1);
	    aurora_reset();
            break;
          } 
        else if (s == '-')
          {
            toggle_powerdown(1);
            break;
          }
		else if (s == '=')
          {
            toggle_intloopback(1);
	    aurora_reset();
            break;
          } 	
		else if (s == 'w')
          {
            toggle_nfc_enable(1);
            break;
          } 
        else if (s == 's')
          {
            inc_nfc(1);
            break;
          } 
        else if (s == 'x')
          {
            dec_nfc(1);
            break;
          }     
		else if (s == 'e')
          {
            input_ufc[0]=TRUE;
            break;
          }
		else if (s == 'r')
          {
            input_ufc[1]=TRUE;
            break;
          }
        else if (s == ' ')
          {
            aurora_reset();
	    drop_frames0=0;
	    drop_frames1=0;
	    drop_words0=0;
	    drop_words1=0;
	    
	    tx_words0=0;
	    tx_words1=0;
	    rx_words0=0;
	    rx_words1=0;
	    tx_frames0=0;
	    tx_frames1=0;
	    rx_frames0=0;
	    rx_frames1=0;
	    
	    tx_words0_high=0;
	    tx_words1_high=0;
	    rx_words0_high=0;
	    rx_words1_high=0;
	    tx_frames0_high=0;
	    tx_frames1_high=0;
	    rx_frames0_high=0;
	    rx_frames1_high=0;
	    tx_words0_full=0;
	    tx_words1_full=0;
	    rx_words0_full=0;
	    rx_words1_full=0;
	    tx_frames0_full=0;
	    tx_frames1_full=0;
	    rx_frames0_full=0;
	    rx_frames1_full=0;
	    
	    tx_words0_int=0;
	    tx_words1_int=0;
	    rx_words0_int=0;
	    rx_words1_int=0;
	    tx_frames0_int=0;
	    tx_frames1_int=0;
	    rx_frames0_int=0;
	    rx_frames1_int=0;
	    
	    tx_words0_prv=0;
	    tx_words1_prv=0;
	    rx_words0_prv=0;
	    rx_words1_prv=0;
	    tx_frames0_prv=0;
	    tx_frames1_prv=0;
	    rx_frames0_prv=0;
	    rx_frames1_prv=0;
	    
	    hard_err_cnt0=0;
	    soft_err_cnt0=0;  
	    framing_err_cnt0=0;
	    cc_cnt0=0;
	    hard_err_cnt1=0;
	    soft_err_cnt1=0;  
	    framing_err_cnt1=0;
	    cc_cnt1=0;
	    
	    delta_cycles=0;
	    
            break;
          }
        i ++;
        usleep(10);   // add some delays to make up for 4 seconds interval
      }
  }
  if( (hard_err_cnt0 == 0 )  && (soft_err_cnt0 == 0) && (framing_err_cnt0 == 0) &&  ( hard_err_cnt1 == 0 )&&  ( soft_err_cnt1 == 0)  &&	(framing_err_cnt1 == 0)  && mgt_linked[0] && mgt_linked[1] && (failed_link == 0) ) {

		 if(print_mode){
		 print("\r\n\r\nSATA loopback test PASSED. \r\n\r\n");
		 }
		else{
		  if(CLOCK_SEL[0] == TRUE){
          print("a.pass\n");
		  }
		  else
          print("b.pass\n");
		  }
	} 
  else{
  		  if(print_mode){
		   print("\r\n\r\nSATA loop back test has ERRORS or Failed to establish link!!\r\nCheck the SATA cable is connected to the correct ports.\r\n");
			}
			else{
			if(CLOCK_SEL[0] == TRUE){
            print("a.fail\n");
		    }
		    else
            print("b.fail\n");
		    }
		}
  if(print_mode){
  print("\r\nType q to retrun to previous menu.\r\n");
  do {
      s = uartRead();
      } while( s  != 'q');
    }
	 
}

LWORD read_dword_data(int sel)
{
 LWORD c;  // 32 bits
 LWORD addr;
 addr = BERT_BaseAddress + (sel-1)*4;
 c = XIo_In32(addr);
 return c;
}

void write_lsb_word_data(WORD d, WORD offset)
{
   XIo_Out16(BERT_BaseAddress + (offset<<2) + 0x2, d);
}

void write_msb_word_data(WORD d, WORD offset)
{
   XIo_Out16(BERT_BaseAddress + (offset<<2), d);
}

void write_lword_data(LWORD d, WORD offset)
{
  XIo_Out32(BERT_BaseAddress + (offset<<2), d);
}

LWORD read_lword_data(WORD offset)
{
 LWORD c;  // 32 bits
 LWORD addr;
 addr = BERT_BaseAddress + (offset<<2);
 c = XIo_In32(addr);
 return c;
}

void toggle_pattern(int sel)
{
  Pattern[sel] = ! Pattern[sel];
  if (sel == 0)
   {
   if (Pattern[0])
     { dip_switch1 = dip_switch1 | 0x0002; }
   else
     { dip_switch1 = dip_switch1 & 0xFFFD; }
   }
  else
   {
   if (Pattern[1])
     { dip_switch1 = dip_switch1 | 0x0200; }
   else
     { dip_switch1 = dip_switch1 & 0xFDFF; }
   }
   write_lsb_word_data (dip_switch1, 0);
}

void toggle_randfrmszen(int sel)
{
  RandFrmSzEn[sel] = !RandFrmSzEn[sel];
  if (sel == 0)
    {
      if (RandFrmSzEn[0])
	{ dip_switch1 = dip_switch1 | 0x0004;  }
      else
	{ dip_switch1 = dip_switch1 & 0xFFFB; }
    }
  else
    {
      if (RandFrmSzEn[1])
	{ dip_switch1 = dip_switch1 | 0x0400;  }
      else
      { dip_switch1 = dip_switch1 & 0xFBFF; }
    }
  write_lsb_word_data (dip_switch1, 0);
}

void toggle_ploopback(int sel)  // parallel loopback
{
   pLoopback[sel] = !pLoopback[sel];
   if (sel == 0)
    {
      if (pLoopback[0])
      { dip_switch1 = dip_switch1 | 0x0008;  }
      else
      { dip_switch1 = dip_switch1 & 0xFFF7; }
    }
   else
    {
      if (pLoopback[1])
      { dip_switch1 = dip_switch1 | 0x0800;  }
      else
      { dip_switch1 = dip_switch1 & 0xF7FF; }
    }
   write_lsb_word_data (dip_switch1, 0);
}

void toggle_sloopback(int sel)  // serial loopback
{
   sLoopback[sel] = !sLoopback[sel];
   if (sel == 0)
    {
      if (sLoopback[0])
      { dip_switch1 = dip_switch1 | 0x0010;  }
      else
      { dip_switch1 = dip_switch1 & 0xFFEF; }
    }
   else
    {
      if (sLoopback[1])
      { dip_switch1 = dip_switch1 | 0x1000;  }
      else
      { dip_switch1 = dip_switch1 & 0xEFFF; }
    }
   write_lsb_word_data (dip_switch1, 0);
}

void toggle_powerdown(int sel)
{
  PowerDown[sel] = ! PowerDown[sel];
  if (sel == 0)
   {
   if (PowerDown[0])
     { dip_switch1 = dip_switch1 | 0x0020; }
   else
     { dip_switch1 = dip_switch1 & 0xFFDF; }
   }
  else
   {
   if (PowerDown[1])
     { dip_switch1 = dip_switch1 | 0x2000; }
   else
     { dip_switch1 = dip_switch1 & 0xDFFF; }
   }
   write_lsb_word_data (dip_switch1, 0);
}

void toggle_intloopback(int sel)  
{
   IntLoopback[sel] = !IntLoopback[sel];
   if (sel == 0)
    {
      if (IntLoopback[0])
      { dip_switch1 = dip_switch1 | 0x0040;  }
      else
      { dip_switch1 = dip_switch1 & 0xFFBF; }
    }
   else
    {
      if (IntLoopback[1])
      { dip_switch1 = dip_switch1 | 0x4000;  }
      else
      { dip_switch1 = dip_switch1 & 0xBFFF; }
    }
   write_lsb_word_data (dip_switch1, 0);
}

void toggle_nfc_enable(int sel)  
{
   nfc_enable[sel] = !nfc_enable[sel];
   if (sel == 0)
    {
      if (nfc_enable[0])
      { dip_switch1 = dip_switch1 | 0x0080;  }
      else
      { dip_switch1 = dip_switch1 & 0xFF7F; }
    }
   else
    {
      if (nfc_enable[1])
      { dip_switch1 = dip_switch1 | 0x8000;  }
      else
      { dip_switch1 = dip_switch1 & 0x7FFF; }
    }
   write_lsb_word_data (dip_switch1, 0);
}

void inc_nfc(int sel)  
{
   if (sel == 0)
    {
      NFC_NB[0] = (nfc_number)>>8;
      NFC_NB[0] = NFC_NB[0] & 0x0f;
      if (NFC_NB[0]==0x0f)
	NFC_NB[0]=0x00;
      else if (NFC_NB[0]==0x08)
	NFC_NB[0]=0x0f;
      else
	NFC_NB[0]++;
      nfc_number = nfc_number & 0xF0FF;
      nfc_number = nfc_number | (NFC_NB[0]<<8);
    }
   else
    {
      NFC_NB[1] = (nfc_number)>>12;
      if (NFC_NB[1]==0x0f)
	NFC_NB[1]=0x00;
      else if (NFC_NB[1]==0x08)
	NFC_NB[1]=0x0f;
      else
	NFC_NB[1]++;
      nfc_number = nfc_number & 0x0FFF;
      nfc_number = nfc_number | (NFC_NB[1]<<12);
    }
   write_msb_word_data (nfc_number, 0);
}

void dec_nfc(int sel)  
{
   if (sel == 0)
    {    
      NFC_NB[0] = (nfc_number)>>8;
      NFC_NB[0] = NFC_NB[0] & 0x0f;
      if (NFC_NB[0]==0x00)
	NFC_NB[0]=0x0f;
      else if (NFC_NB[0]==0x0f)
	NFC_NB[0]=0x08;
      else
	NFC_NB[0]--;
      nfc_number = nfc_number & 0xF0FF;
      nfc_number = nfc_number | (NFC_NB[0]<<8);
    }
   else
    {
      NFC_NB[1] = (nfc_number)>>12;
      if (NFC_NB[1]==0x00)
	NFC_NB[1]=0x0f;
      else if (NFC_NB[1]==0x0f)
	NFC_NB[1]=0x08;
      else
	NFC_NB[1]--;
      nfc_number = nfc_number & 0x0FFF;
      nfc_number = nfc_number | (NFC_NB[1]<<12);
    }
   write_msb_word_data (nfc_number, 0);
}

void write_ufc (int sel)  
{
  write_lword_data (UFC_TX_MSG[sel][0], 18+(sel<<2));
  write_lword_data (UFC_TX_MSG[sel][1], 19+(sel<<2));
  write_lword_data (UFC_TX_MSG[sel][2], 20+(sel<<2));
  write_lword_data (UFC_TX_MSG[sel][3], 21+(sel<<2)); 
}

void trigger_ufc (int sel)
{
  if (sel == 0)
    {
      dip_switch4 = dip_switch4 | 0x0004;
      write_msb_word_data (dip_switch4, 0);  
      usleep(100000);//was 100000, and was commented out??
      dip_switch4 = dip_switch4 & 0xFFFB;
      write_msb_word_data (dip_switch4, 0);
    }
  else
    {
      dip_switch4 = dip_switch4 | 0x0001;
      write_msb_word_data (dip_switch4, 0);
      usleep(100000);
      dip_switch4 = dip_switch4 & 0xFFFE;
      write_msb_word_data (dip_switch4, 0);
    }
} 

void read_ufc (int sel)
{
  int i;

  for(i=0;i<4;i++)
    {
      UFC_RX_MSG[sel][i] = read_lword_data(18+i+(sel<<2));
    }
}

int reset_sequence(int print_mode)
{

 int j;
 LWORD bert_status;
 BOOL mgt_linked[2];
 int failed_link = 0;

   if(print_mode){
   print("\r\nResetting MGTs...\t");
   }
   master_reset_pulse();
   usleep(500000);
   aurora_reset();
   usleep(500000);   // wait a while

   for ( j = 0; j <= 4; j ++) // check for link up 4 times
   {
    bert_status = read_dword_data(17); // sample again on the bert status
    mgt_linked[0] = get_bool(bert_status & 0x00000008);
    mgt_linked[1] = get_bool(bert_status & 0x00000800);

    if (mgt_linked[0] || mgt_linked[1]){
      return(failed_link);
	  }
    else
      {
	aurora_reset();
	usleep(500000);
      }
   }
   if(print_mode){
     print("done!\r\n");
     }
   if (!mgt_linked[0] && !mgt_linked[1])
    {
	  failed_link = 1;
       if(print_mode){
       xil_printf("Warning: Couldn't establish links on both cores.\r\n");
       xil_printf("Press any key to continue...\r\n");
	   uartRead();
	   }
     return(failed_link);  
    }

   rx_words_prv0 = 0;
   rx_words0_nodiv = 0;
   rx_words_prv1 = 0;
   rx_words1_nodiv = 0;

   XTime_SetTime(0);
   usleep(1000000);  // wait a while to get some frames

}

void master_reset_pulse()
{
  // pulse master reset
  push_button = push_button & (0xFFF0);
  push_button = push_button | (0x0001);
  write_lsb_word_data (push_button, 1);
  push_button = push_button & (0xFFF0);
  write_lsb_word_data (push_button, 1);
}

void aurora_reset()
{
  // pulse aurora resets
  push_button = push_button & (0xFFF0);
  push_button = push_button | (0x0006);
  write_lsb_word_data (push_button, 1);
  usleep(100000); // wait a while
  push_button = push_button & (0xFFF0);
  write_lsb_word_data (push_button, 1);
}

void change_frame_size()
{
 
 dip_switch2 = FRAME_SIZE1;
 write_lsb_word_data (dip_switch2, 2);
 dip_switch3 = FRAME_SIZE2;
 write_msb_word_data (dip_switch3, 2); // set the pattern 1
  
 usleep(100000); // wait a while
}

void change_clock_source()
{
  if (CLOCK_SEL[0])
    dip_switch4 = dip_switch4 | 0x0008;
  else
    dip_switch4 = dip_switch4 & 0xFFF7;
  
  if (CLOCK_SEL[1])
    dip_switch4 = dip_switch4 | 0x0002;
  else
    dip_switch4 = dip_switch4 & 0xFFFD;
  
  write_msb_word_data (dip_switch4, 0); 
  usleep(100000); // wait a while
}

LWORD get_cycles()
{
  LWORD cyc;
  XTime_GetTime(stop_cycle); // stop timer

   if (stop_cycle[0] != 0)
   { cyc = 0;
   }
   else
   { cyc = stop_cycle[1];}

   return cyc;
}

//debug was static
BOOL get_bool(LWORD v)
{
  BOOL t;
  if (v == 0)
     { t = FALSE;}
  else
     { t = TRUE;}
  return t;

}

char int2char(LWORD a) {
  char ret;
  
  switch (a) {
  case 0:ret = '0'; break;
  case 1:ret = '1'; break;
  case 2:ret = '2'; break;
  case 3:ret = '3'; break;	
  case 4:ret = '4'; break;
  case 5:ret = '5'; break;
  case 6:ret = '6'; break;
  case 7:ret = '7'; break;      
  case 8:ret = '8'; break;
  case 9:ret = '9'; break;
  }
  return ret;
}

LWORD pow(LWORD x, LWORD y) {
  LWORD ret=1;
  int i;
  for(i=0;i<y;i++)
    ret = ret*x;
  return ret;
  }

void print_ufc_msg(int a)
{
  LWORD temp_lword=0x00000000;
  
  int l, m;
  char s;
  char ufc_char=0x20;
  int num_chars=0;
  char str[2];
  
  str[1] = '\0';

  if (input_ufc[a])
    {
      UFC_TX_MSG[a][0] = 0x20202020; //initialize to all spaces
      UFC_TX_MSG[a][1] = 0x20202020;
      UFC_TX_MSG[a][2] = 0x20202020;
      UFC_TX_MSG[a][3] = 0x20202020;
      xil_printf("| Aurora%d UFC TX | ", a);
      for (m=0;m<4;m++)
	{
	  for (l=0;l<4;l++)
	    {
	      ufc_char = uartRead();
	      if (ufc_char == 0x0D) //return character
		break;
	      str[0] = ufc_char;
	      xil_printf("%s",str); // echo the input
	      temp_lword = temp_lword<<8;
	      temp_lword = temp_lword | ufc_char;
	    }
	  UFC_TX_MSG[a][m] = temp_lword;
	  temp_lword = 0;
	  if (ufc_char == 0x0D) //return character
	    break;
	}
      xil_printf("\r\n");
      input_ufc[a]=FALSE;
      write_ufc(a);
      trigger_ufc(a);
    }
  else
    {
      xil_printf("| Aurora%d UFC TX | ", a);
      print_lword(UFC_TX_MSG[a][0]);
      print_lword(UFC_TX_MSG[a][1]);
      print_lword(UFC_TX_MSG[a][2]);
      print_lword(UFC_TX_MSG[a][3]);
      xil_printf("\r\n");
    }	
  read_ufc(a);
  xil_printf("| Aurora%d UFC RX | ", a);
  print("\r\n");
}

void print_char_array(DLWORD a) {
  LWORD q;
  LWORD div;
  char *ret;
  int i;
  BOOL flag=FALSE;
  

  ret = (char *)malloc(11*sizeof(char));  

  for(i=10;i>=0;i--) {
    div = pow(10,i);
    q = a/div;
    a = a-div*q;
    if (q>0)
      flag=TRUE;
    if (q==0 && i>0 && flag==FALSE)
      xil_printf(" ");
    else
      xil_printf("%d", q);
  }

  free(ret);

} 

void print_lword(LWORD a) {
  int i; 
  char str[2];
  str[1] = '\0';
  for(i=3;i>=0;i--)
    {
      str[0] = (char)(a>>8*i);
      xil_printf("%s",str);
    }
} 

void force_powerdown(int sel)
{
  if (sel == 0)
   {
   // force status bit to 0 
   PowerDown[0] = 1;
   PowerDown[1] = 1;   
   dip_switch1 = dip_switch1 | 0x0020;
   write_lsb_word_data (dip_switch1, 0);
   dip_switch1 = dip_switch1 | 0x2000;
   write_lsb_word_data (dip_switch1, 0);
   }     
   else
   {
   // force status bit to 1 

   PowerDown[0] = 0;
   PowerDown[1] = 0;
   dip_switch1 = dip_switch1 & 0xFFDF;
   write_lsb_word_data (dip_switch1, 0);
   dip_switch1 = dip_switch1 & 0xDFFF;
   write_lsb_word_data (dip_switch1, 0);
   }
}

void loop_0to1_test(int print_mode, int end_frame_num)
{
	  CLOCK_SEL[0] = TRUE;
	  CLOCK_SEL[1] = TRUE;
	  change_clock_source();
	  // turn on the mgts
  	  force_powerdown(1); 
      aurora_test(print_mode, end_frame_num);
	  CLOCK_SEL[0] = FALSE;
	  CLOCK_SEL[1] = FALSE;
	  change_clock_source();
	  // turn off the mgts
      force_powerdown(0);
} 
void loop_1to2_test(int print_mode, int end_frame_num)
{
	 // turn on the mgts
  	 force_powerdown(1);
     aurora_test(print_mode, end_frame_num);
	 // turn off the mgts
     force_powerdown(0);
}