//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2004 Xilinx, Inc.
//     All rights reserved.
//
/*
-------------------------------------------------------------------------------
   Title      : EXPANSION PORT TEST
   Project    : XUP Virtex-II Pro Development System
-------------------------------------------------------------------------------

   File       : EXPANSION_PORTS.v
   Company    : Xilinx, Inc.
   Created    : 2004/08/12
   Last Update: 2004/08/12
   Copyright  : (c) Xilinx Inc, 2004
-------------------------------------------------------------------------------
   Uses       : 
-------------------------------------------------------------------------------
   Used by    : HW_BIST.v
-------------------------------------------------------------------------------
    Description: 	This design creates a walking one across the expansion ports.
					There should be a ~20nS pulse every 1.6uS


	Conventions:
		All external port signals are UPPER CASE.
		All internal signals are LOWER CASE and are active HIGH.

-------------------------------------------------------------------------------
*/

module  EXPANSION_PORT_TEST
(
_100MHz_clock,
reset,
hw_test_enable,
sft_test_enable,
sft_test_data,
sft_test_clk,
sft_test_rd_data_lo,
sft_test_rd_data_hi,
EXPANSION_PORT,
HIGH_SPEED_PORT
);

input	_100MHz_clock;			// 100MHz LVTTL SYSTEM CLOCK
input	reset;
input	hw_test_enable;
input	sft_test_enable;
input	sft_test_data;
input	sft_test_clk;
output [0:31] sft_test_rd_data_lo;
output [0:31] sft_test_rd_data_hi;
output [79:0] EXPANSION_PORT;	// header pins and low-speed Digilent port
output [42:0] HIGH_SPEED_PORT;	// high-speed Digilent port


reg [79:0] counter0;
reg [52:0] counter1;

reg _50mhz_clock;
wire oe_hw_test;
wire [52:0] mux_out;
wire [52:0] oe_enable_dual;

assign oe_hw_test = !(hw_test_enable);
assign sft_test_rd_data_hi[0:10] = 11'b0;   // tie unused read bits to zero

// create a 50MHz clock
always @ (posedge _100MHz_clock or posedge reset) begin
	if (reset) begin
		_50mhz_clock <= 1'b0;
		end
	else begin
		_50mhz_clock <= ~_50mhz_clock;
		end
	end

// create the walking one test pattern
always @ (posedge _100MHz_clock or posedge reset) begin
	if (reset) begin
		counter0[79:0] <= 80'h00000001;
		end
	else if (_50mhz_clock) begin
		counter0[79:1] <= counter0[78:0];
		counter0[0] <= counter0[79];
		end
	end


    // create the software controlled walking one test pattern
always @ (posedge sft_test_clk or posedge reset) begin
	if (reset) begin
		counter1[52:0] <= 53'b0;
		end
	else  begin
		counter1[52:0] <= {counter1[51:0] , sft_test_data}  ;
		end
	end

//assign EXPANSION_PORT[79:0] = counter0[79:0];
//assign HIGH_SPEED_PORT[42:0] = counter0[42:0];

// expansion pins only driven by hardware test no read
IOBUF expan_port_0(.O(), .IO(EXPANSION_PORT[0]), .I(counter0[0]), .T(oe_hw_test)); 
IOBUF expan_port_1(.O(), .IO(EXPANSION_PORT[1]), .I(counter0[1]), .T(oe_hw_test)); 
IOBUF expan_port_2(.O(), .IO(EXPANSION_PORT[2]), .I(counter0[2]), .T(oe_hw_test)); 
IOBUF expan_port_3(.O(), .IO(EXPANSION_PORT[3]), .I(counter0[3]), .T(oe_hw_test)); 
IOBUF expan_port_4(.O(), .IO(EXPANSION_PORT[4]), .I(counter0[4]), .T(oe_hw_test)); 
IOBUF expan_port_5(.O(), .IO(EXPANSION_PORT[5]), .I(counter0[5]), .T(oe_hw_test)); 
IOBUF expan_port_6(.O(), .IO(EXPANSION_PORT[6]), .I(counter0[6]), .T(oe_hw_test)); 
IOBUF expan_port_7(.O(), .IO(EXPANSION_PORT[7]), .I(counter0[7]), .T(oe_hw_test)); 
IOBUF expan_port_40(.O(), .IO(EXPANSION_PORT[40]), .I(counter0[40]), .T(oe_hw_test)); 
IOBUF expan_port_41(.O(), .IO(EXPANSION_PORT[41]), .I(counter0[41]), .T(oe_hw_test)); 
IOBUF expan_port_42(.O(), .IO(EXPANSION_PORT[42]), .I(counter0[42]), .T(oe_hw_test)); 
IOBUF expan_port_43(.O(), .IO(EXPANSION_PORT[43]), .I(counter0[43]), .T(oe_hw_test));
IOBUF expan_port_44(.O(), .IO(EXPANSION_PORT[44]), .I(counter0[44]), .T(oe_hw_test));
IOBUF expan_port_77(.O(), .IO(EXPANSION_PORT[77]), .I(counter0[77]), .T(oe_hw_test));
IOBUF expan_port_78(.O(), .IO(EXPANSION_PORT[78]), .I(counter0[78]), .T(oe_hw_test));
IOBUF expan_port_79(.O(), .IO(EXPANSION_PORT[79]), .I(counter0[79]), .T(oe_hw_test));
IOBUF hs_expan_42(.O(), .IO(HIGH_SPEED_PORT[42]), .I(counter0[42]), .T(oe_hw_test));

 
// expansion pins  driven by both  hardware and software test
assign mux_out[0] = hw_test_enable ? counter0[8] : counter1[0];
assign oe_enable_dual[0] = !(counter1[0] | hw_test_enable);
IOBUF expan_port_8(.O(), .IO(EXPANSION_PORT[8]), .I(mux_out[0]), .T(oe_enable_dual[0]));
assign mux_out[1] = hw_test_enable ? counter0[9] : counter1[1];
assign oe_enable_dual[1] = !(counter1[1] | hw_test_enable);
IOBUF expan_port_9(.O(), .IO(EXPANSION_PORT[9]), .I(mux_out[1]), .T(oe_enable_dual[1]));
assign mux_out[2] = hw_test_enable ? counter0[11] : counter1[2];
assign oe_enable_dual[2] = !(counter1[2] | hw_test_enable);
IOBUF expan_port_11(.O(), .IO(EXPANSION_PORT[11]), .I(mux_out[2]), .T(oe_enable_dual[2]));
assign mux_out[3] = hw_test_enable ? counter0[13] : counter1[3];
assign oe_enable_dual[3] = !(counter1[3] | hw_test_enable);
IOBUF expan_port_13(.O(), .IO(EXPANSION_PORT[13]), .I(mux_out[3]), .T(oe_enable_dual[3]));
assign mux_out[4] = hw_test_enable ? counter0[15] : counter1[4];
assign oe_enable_dual[4] = !(counter1[4] | hw_test_enable);
IOBUF expan_port_15(.O(), .IO(EXPANSION_PORT[15]), .I(mux_out[4]), .T(oe_enable_dual[4]));
assign mux_out[5] = hw_test_enable ? counter0[17] : counter1[5];
assign oe_enable_dual[5] = !(counter1[5] | hw_test_enable);
IOBUF expan_port_17(.O(), .IO(EXPANSION_PORT[17]), .I(mux_out[5]), .T(oe_enable_dual[5]));
assign mux_out[6] = hw_test_enable ? counter0[19] : counter1[6];
assign oe_enable_dual[6] = !(counter1[6] | hw_test_enable);
IOBUF expan_port_19(.O(), .IO(EXPANSION_PORT[19]), .I(mux_out[6]), .T(oe_enable_dual[6]));
assign mux_out[7] = hw_test_enable ? counter0[21] : counter1[7];
assign oe_enable_dual[7] = !(counter1[7] | hw_test_enable);
IOBUF expan_port_21(.O(), .IO(EXPANSION_PORT[21]), .I(mux_out[7]), .T(oe_enable_dual[7]));
assign mux_out[8] = hw_test_enable ? counter0[23] : counter1[8];
assign oe_enable_dual[8] = !(counter1[8] | hw_test_enable);
IOBUF expan_port_23(.O(), .IO(EXPANSION_PORT[23]), .I(mux_out[8]), .T(oe_enable_dual[8]));
assign mux_out[9] = hw_test_enable ? counter0[25] : counter1[9];
assign oe_enable_dual[9] = !(counter1[9] | hw_test_enable);
IOBUF expan_port_25(.O(), .IO(EXPANSION_PORT[25]), .I(mux_out[9]), .T(oe_enable_dual[9]));
assign mux_out[10] = hw_test_enable ? counter0[27] : counter1[10];
assign oe_enable_dual[10] = !(counter1[10] | hw_test_enable);
IOBUF expan_port_27(.O(), .IO(EXPANSION_PORT[27]), .I(mux_out[10]), .T(oe_enable_dual[10]));
assign mux_out[11] = hw_test_enable ? counter0[29] : counter1[11];
assign oe_enable_dual[11] = !(counter1[11] | hw_test_enable);
IOBUF expan_port_29(.O(), .IO(EXPANSION_PORT[29]), .I(mux_out[11]), .T(oe_enable_dual[11]));
assign mux_out[12] = hw_test_enable ? counter0[31] : counter1[12];
assign oe_enable_dual[12] = !(counter1[12] | hw_test_enable);
IOBUF expan_port_31(.O(), .IO(EXPANSION_PORT[31]), .I(mux_out[12]), .T(oe_enable_dual[12]));
assign mux_out[13] = hw_test_enable ? counter0[33] : counter1[13];
assign oe_enable_dual[13] = !(counter1[13] | hw_test_enable);
IOBUF expan_port_33(.O(), .IO(EXPANSION_PORT[33]), .I(mux_out[13]), .T(oe_enable_dual[13]));
assign mux_out[14] = hw_test_enable ? counter0[35] : counter1[14];
assign oe_enable_dual[14] = !(counter1[14] | hw_test_enable);
IOBUF expan_port_35(.O(), .IO(EXPANSION_PORT[35]), .I(mux_out[14]), .T(oe_enable_dual[14]));
assign mux_out[15] = hw_test_enable ? counter0[37] : counter1[15];
assign oe_enable_dual[15] = !(counter1[15] | hw_test_enable);
IOBUF expan_port_37(.O(), .IO(EXPANSION_PORT[37]), .I(mux_out[15]), .T(oe_enable_dual[15]));
assign mux_out[16] = hw_test_enable ? counter0[45] : counter1[16];
assign oe_enable_dual[16] = !(counter1[16] | hw_test_enable);
IOBUF expan_port_45(.O(), .IO(EXPANSION_PORT[45]), .I(mux_out[16]), .T(oe_enable_dual[16]));
assign mux_out[17] = hw_test_enable ? counter0[46] : counter1[17];
assign oe_enable_dual[17] = !(counter1[17] | hw_test_enable);
IOBUF expan_port_46(.O(), .IO(EXPANSION_PORT[46]), .I(mux_out[17]), .T(oe_enable_dual[17]));
assign mux_out[18] = hw_test_enable ? counter0[47] : counter1[18];
assign oe_enable_dual[18] = !(counter1[18] | hw_test_enable);
IOBUF expan_port_47(.O(), .IO(EXPANSION_PORT[47]), .I(mux_out[18]), .T(oe_enable_dual[18]));
assign mux_out[19] = hw_test_enable ? counter0[49] : counter1[19];
assign oe_enable_dual[19] = !(counter1[19] | hw_test_enable);
IOBUF expan_port_49(.O(), .IO(EXPANSION_PORT[49]), .I(mux_out[19]), .T(oe_enable_dual[19]));
assign mux_out[20] = hw_test_enable ? counter0[51] : counter1[20];
assign oe_enable_dual[20] = !(counter1[20] | hw_test_enable);
IOBUF expan_port_51(.O(), .IO(EXPANSION_PORT[51]), .I(mux_out[20]), .T(oe_enable_dual[20]));
assign mux_out[21] = hw_test_enable ? counter0[53] : counter1[21];
assign oe_enable_dual[21] = !(counter1[21] | hw_test_enable);
IOBUF expan_port_53(.O(), .IO(EXPANSION_PORT[53]), .I(mux_out[21]), .T(oe_enable_dual[21]));
assign mux_out[22] = hw_test_enable ? counter0[55] : counter1[22];
assign oe_enable_dual[22] = !(counter1[22] | hw_test_enable);
IOBUF expan_port_55(.O(), .IO(EXPANSION_PORT[55]), .I(mux_out[22]), .T(oe_enable_dual[22]));
assign mux_out[23] = hw_test_enable ? counter0[57] : counter1[23];
assign oe_enable_dual[23] = !(counter1[23] | hw_test_enable);
IOBUF expan_port_57(.O(), .IO(EXPANSION_PORT[57]), .I(mux_out[23]), .T(oe_enable_dual[23]));
assign mux_out[24] = hw_test_enable ? counter0[59] : counter1[24];
assign oe_enable_dual[24] = !(counter1[24] | hw_test_enable);
IOBUF expan_port_59(.O(), .IO(EXPANSION_PORT[59]), .I(mux_out[24]), .T(oe_enable_dual[24]));
assign mux_out[25] = hw_test_enable ? counter0[61] : counter1[25];
assign oe_enable_dual[25] = !(counter1[25] | hw_test_enable);
IOBUF expan_port_61(.O(), .IO(EXPANSION_PORT[61]), .I(mux_out[25]), .T(oe_enable_dual[25]));
assign mux_out[26] = hw_test_enable ? counter0[63] : counter1[26];
assign oe_enable_dual[26] = !(counter1[26] | hw_test_enable);
IOBUF expan_port_63(.O(), .IO(EXPANSION_PORT[63]), .I(mux_out[26]), .T(oe_enable_dual[26]));
assign mux_out[27] = hw_test_enable ? counter0[65] : counter1[27];
assign oe_enable_dual[27] = !(counter1[27] | hw_test_enable);
IOBUF expan_port_65(.O(), .IO(EXPANSION_PORT[65]), .I(mux_out[27]), .T(oe_enable_dual[27]));
assign mux_out[28] = hw_test_enable ? counter0[67] : counter1[28];
assign oe_enable_dual[28] = !(counter1[28] | hw_test_enable);
IOBUF expan_port_67(.O(), .IO(EXPANSION_PORT[67]), .I(mux_out[28]), .T(oe_enable_dual[28]));
assign mux_out[29] = hw_test_enable ? counter0[69] : counter1[29];
assign oe_enable_dual[29] = !(counter1[29] | hw_test_enable);
IOBUF expan_port_69(.O(), .IO(EXPANSION_PORT[69]), .I(mux_out[29]), .T(oe_enable_dual[29]));
assign mux_out[30] = hw_test_enable ? counter0[71] : counter1[30];
assign oe_enable_dual[30] = !(counter1[30] | hw_test_enable);
IOBUF expan_port_71(.O(), .IO(EXPANSION_PORT[71]), .I(mux_out[30]), .T(oe_enable_dual[30]));
assign mux_out[31] = hw_test_enable ? counter0[73] : counter1[31];
assign oe_enable_dual[31] = !(counter1[31] | hw_test_enable);
IOBUF expan_port_73(.O(), .IO(EXPANSION_PORT[73]), .I(mux_out[31]), .T(oe_enable_dual[31]));
////////////////////////////////////////////////////////////////////////////////////////////
// start of hi speed ports with dual drive
assign mux_out[32] = hw_test_enable ? counter0[0] : counter1[32];
assign oe_enable_dual[32] = !(counter1[32] | hw_test_enable);
IOBUF hs_expan_0(.O(), .IO(HIGH_SPEED_PORT[0]), .I(mux_out[32]), .T(oe_enable_dual[32]));
assign mux_out[33] = hw_test_enable ? counter0[2] : counter1[33];
assign oe_enable_dual[33] = !(counter1[33] | hw_test_enable);
IOBUF hs_expan_2(.O(), .IO(HIGH_SPEED_PORT[2]), .I(mux_out[33]), .T(oe_enable_dual[33]));
assign mux_out[34] = hw_test_enable ? counter0[4] : counter1[34];
assign oe_enable_dual[34] = !(counter1[34] | hw_test_enable);
IOBUF hs_expan_4(.O(), .IO(HIGH_SPEED_PORT[4]), .I(mux_out[34]), .T(oe_enable_dual[34]));
assign mux_out[35] = hw_test_enable ? counter0[6] : counter1[35];
assign oe_enable_dual[35] = !(counter1[35] | hw_test_enable);
IOBUF hs_expan_6(.O(), .IO(HIGH_SPEED_PORT[6]), .I(mux_out[35]), .T(oe_enable_dual[35]));
assign mux_out[36] = hw_test_enable ? counter0[8] : counter1[36];
assign oe_enable_dual[36] = !(counter1[36] | hw_test_enable);
IOBUF hs_expan_8(.O(), .IO(HIGH_SPEED_PORT[8]), .I(mux_out[36]), .T(oe_enable_dual[36]));
assign mux_out[37] = hw_test_enable ? counter0[10] : counter1[37];
assign oe_enable_dual[37] = !(counter1[37] | hw_test_enable);
IOBUF hs_expan_10(.O(), .IO(HIGH_SPEED_PORT[10]), .I(mux_out[37]), .T(oe_enable_dual[37]));
assign mux_out[38] = hw_test_enable ? counter0[12] : counter1[38];
assign oe_enable_dual[38] = !(counter1[38] | hw_test_enable);
IOBUF hs_expan_12(.O(), .IO(HIGH_SPEED_PORT[12]), .I(mux_out[38]), .T(oe_enable_dual[38]));
assign mux_out[39] = hw_test_enable ? counter0[14] : counter1[39];
assign oe_enable_dual[39] = !(counter1[39] | hw_test_enable);
IOBUF hs_expan_14(.O(), .IO(HIGH_SPEED_PORT[14]), .I(mux_out[39]), .T(oe_enable_dual[39]));
assign mux_out[40] = hw_test_enable ? counter0[16] : counter1[40];
assign oe_enable_dual[40] = !(counter1[40] | hw_test_enable);
IOBUF hs_expan_16(.O(), .IO(HIGH_SPEED_PORT[16]), .I(mux_out[40]), .T(oe_enable_dual[40]));
assign mux_out[41] = hw_test_enable ? counter0[18] : counter1[41];
assign oe_enable_dual[41] = !(counter1[41] | hw_test_enable);
IOBUF hs_expan_18(.O(), .IO(HIGH_SPEED_PORT[18]), .I(mux_out[41]), .T(oe_enable_dual[41]));
assign mux_out[42] = hw_test_enable ? counter0[20] : counter1[42];
assign oe_enable_dual[42] = !(counter1[42] | hw_test_enable);
IOBUF hs_expan_20(.O(), .IO(HIGH_SPEED_PORT[20]), .I(mux_out[42]), .T(oe_enable_dual[42]));
assign mux_out[43] = hw_test_enable ? counter0[22] : counter1[43];
assign oe_enable_dual[43] = !(counter1[43] | hw_test_enable);
IOBUF hs_expan_22(.O(), .IO(HIGH_SPEED_PORT[22]), .I(mux_out[43]), .T(oe_enable_dual[43]));
assign mux_out[44] = hw_test_enable ? counter0[24] : counter1[44];
assign oe_enable_dual[44] = !(counter1[44] | hw_test_enable);
IOBUF hs_expan_24(.O(), .IO(HIGH_SPEED_PORT[24]), .I(mux_out[44]), .T(oe_enable_dual[44]));
assign mux_out[45] = hw_test_enable ? counter0[26] : counter1[45];
assign oe_enable_dual[45] = !(counter1[45] | hw_test_enable);
IOBUF hs_expan_26(.O(), .IO(HIGH_SPEED_PORT[26]), .I(mux_out[45]), .T(oe_enable_dual[45]));
assign mux_out[46] = hw_test_enable ? counter0[28] : counter1[46];
assign oe_enable_dual[46] = !(counter1[46] | hw_test_enable);
IOBUF hs_expan_28(.O(), .IO(HIGH_SPEED_PORT[28]), .I(mux_out[46]), .T(oe_enable_dual[46]));
assign mux_out[47] = hw_test_enable ? counter0[30] : counter1[47];
assign oe_enable_dual[47] = !(counter1[47] | hw_test_enable);
IOBUF hs_expan_30(.O(), .IO(HIGH_SPEED_PORT[30]), .I(mux_out[47]), .T(oe_enable_dual[47]));
assign mux_out[48] = hw_test_enable ? counter0[32] : counter1[48];
assign oe_enable_dual[48] = !(counter1[48] | hw_test_enable);
IOBUF hs_expan_32(.O(), .IO(HIGH_SPEED_PORT[32]), .I(mux_out[48]), .T(oe_enable_dual[48]));
assign mux_out[49] = hw_test_enable ? counter0[34] : counter1[49];
assign oe_enable_dual[49] = !(counter1[49] | hw_test_enable);
IOBUF hs_expan_34(.O(), .IO(HIGH_SPEED_PORT[34]), .I(mux_out[49]), .T(oe_enable_dual[49]));
assign mux_out[50] = hw_test_enable ? counter0[36] : counter1[50];
assign oe_enable_dual[50] = !(counter1[50] | hw_test_enable);
IOBUF hs_expan_36(.O(), .IO(HIGH_SPEED_PORT[36]), .I(mux_out[50]), .T(oe_enable_dual[50]));
assign mux_out[51] = hw_test_enable ? counter0[38] : counter1[51];
assign oe_enable_dual[51] = !(counter1[51] | hw_test_enable);
IOBUF hs_expan_38(.O(), .IO(HIGH_SPEED_PORT[38]), .I(mux_out[51]), .T(oe_enable_dual[51]));
assign mux_out[52] = hw_test_enable ? counter0[40] : counter1[52];
assign oe_enable_dual[52] = !(counter1[52] | hw_test_enable);
IOBUF hs_expan_40(.O(), .IO(HIGH_SPEED_PORT[40]), .I(mux_out[52]), .T(oe_enable_dual[52]));



// expansion pins with hardware test out & software read
IOBUF expan_port_10(.O(sft_test_rd_data_lo[31]), .IO(EXPANSION_PORT[10]), .I(counter0[10]), .T(oe_hw_test)); //in 0
IOBUF expan_port_12(.O(sft_test_rd_data_lo[30]), .IO(EXPANSION_PORT[12]), .I(counter0[12]), .T(oe_hw_test)); // in 1
IOBUF expan_port_14(.O(sft_test_rd_data_lo[29]), .IO(EXPANSION_PORT[14]), .I(counter0[14]), .T(oe_hw_test));  // in 2
IOBUF expan_port_16(.O(sft_test_rd_data_lo[28]), .IO(EXPANSION_PORT[16]), .I(counter0[16]), .T(oe_hw_test));  // in 3
IOBUF expan_port_18(.O(sft_test_rd_data_lo[27]), .IO(EXPANSION_PORT[18]), .I(counter0[18]), .T(oe_hw_test));  // in 4
IOBUF expan_port_20(.O(sft_test_rd_data_lo[26]), .IO(EXPANSION_PORT[20]), .I(counter0[20]), .T(oe_hw_test));  // in 5
IOBUF expan_port_22(.O(sft_test_rd_data_lo[25]), .IO(EXPANSION_PORT[22]), .I(counter0[22]), .T(oe_hw_test));  // in 6
IOBUF expan_port_24(.O(sft_test_rd_data_lo[24]), .IO(EXPANSION_PORT[24]), .I(counter0[24]), .T(oe_hw_test));  // in 7
IOBUF expan_port_26(.O(sft_test_rd_data_lo[23]), .IO(EXPANSION_PORT[26]), .I(counter0[26]), .T(oe_hw_test));  // in 8
IOBUF expan_port_28(.O(sft_test_rd_data_lo[22]), .IO(EXPANSION_PORT[28]), .I(counter0[28]), .T(oe_hw_test));  // in 9
IOBUF expan_port_30(.O(sft_test_rd_data_lo[21]), .IO(EXPANSION_PORT[30]), .I(counter0[30]), .T(oe_hw_test));  // in 10
IOBUF expan_port_32(.O(sft_test_rd_data_lo[20]), .IO(EXPANSION_PORT[32]), .I(counter0[32]), .T(oe_hw_test));  // in 11
IOBUF expan_port_34(.O(sft_test_rd_data_lo[19]), .IO(EXPANSION_PORT[34]), .I(counter0[34]), .T(oe_hw_test));  // in 12 
IOBUF expan_port_36(.O(sft_test_rd_data_lo[18]), .IO(EXPANSION_PORT[36]), .I(counter0[36]), .T(oe_hw_test));  // in 13 
IOBUF expan_port_38(.O(sft_test_rd_data_lo[17]), .IO(EXPANSION_PORT[38]), .I(counter0[38]), .T(oe_hw_test));  // in 14
IOBUF expan_port_39(.O(sft_test_rd_data_lo[16]), .IO(EXPANSION_PORT[39]), .I(counter0[39]), .T(oe_hw_test));  // in 15
IOBUF expan_port_50(.O(sft_test_rd_data_lo[15]), .IO(EXPANSION_PORT[50]), .I(counter0[50]), .T(oe_hw_test));  // in 16
IOBUF expan_port_48(.O(sft_test_rd_data_lo[14]), .IO(EXPANSION_PORT[48]), .I(counter0[48]), .T(oe_hw_test));  // in 17
IOBUF expan_port_52(.O(sft_test_rd_data_lo[13]), .IO(EXPANSION_PORT[52]), .I(counter0[52]), .T(oe_hw_test));  // in 18
IOBUF expan_port_54(.O(sft_test_rd_data_lo[12]), .IO(EXPANSION_PORT[54]), .I(counter0[54]), .T(oe_hw_test));  // in 19
IOBUF expan_port_56(.O(sft_test_rd_data_lo[11]), .IO(EXPANSION_PORT[56]), .I(counter0[56]), .T(oe_hw_test));  // in 20
IOBUF expan_port_58(.O(sft_test_rd_data_lo[10]), .IO(EXPANSION_PORT[58]), .I(counter0[58]), .T(oe_hw_test));  // in 21
IOBUF expan_port_60(.O(sft_test_rd_data_lo[9]), .IO(EXPANSION_PORT[60]), .I(counter0[60]), .T(oe_hw_test));  // in 22
IOBUF expan_port_62(.O(sft_test_rd_data_lo[8]), .IO(EXPANSION_PORT[62]), .I(counter0[62]), .T(oe_hw_test));  // in 23
IOBUF expan_port_64(.O(sft_test_rd_data_lo[7]), .IO(EXPANSION_PORT[64]), .I(counter0[64]), .T(oe_hw_test));  // in 24
IOBUF expan_port_66(.O(sft_test_rd_data_lo[6]), .IO(EXPANSION_PORT[66]), .I(counter0[66]), .T(oe_hw_test));  // in 25
IOBUF expan_port_68(.O(sft_test_rd_data_lo[5]), .IO(EXPANSION_PORT[68]), .I(counter0[68]), .T(oe_hw_test));  // in 26
IOBUF expan_port_70(.O(sft_test_rd_data_lo[4]), .IO(EXPANSION_PORT[70]), .I(counter0[70]), .T(oe_hw_test));  // in 27
IOBUF expan_port_72(.O(sft_test_rd_data_lo[3]), .IO(EXPANSION_PORT[72]), .I(counter0[72]), .T(oe_hw_test));  // in 28
IOBUF expan_port_74(.O(sft_test_rd_data_lo[2]), .IO(EXPANSION_PORT[74]), .I(counter0[74]), .T(oe_hw_test));  // in 29
IOBUF expan_port_76(.O(sft_test_rd_data_lo[1]), .IO(EXPANSION_PORT[76]), .I(counter0[76]), .T(oe_hw_test));  // in 30
IOBUF expan_port_75(.O(sft_test_rd_data_lo[0]), .IO(EXPANSION_PORT[75]), .I(counter0[75]), .T(oe_hw_test));  // in 31

IOBUF hs_expan_1(.O(sft_test_rd_data_hi[31]), .IO(HIGH_SPEED_PORT[1]), .I(counter0[1]), .T(oe_hw_test));  // in 32
IOBUF hs_expan_3(.O(sft_test_rd_data_hi[30]), .IO(HIGH_SPEED_PORT[3]), .I(counter0[3]), .T(oe_hw_test));  // in 33
IOBUF hs_expan_5(.O(sft_test_rd_data_hi[29]), .IO(HIGH_SPEED_PORT[5]), .I(counter0[5]), .T(oe_hw_test));  // in 34 
IOBUF hs_expan_7(.O(sft_test_rd_data_hi[28]), .IO(HIGH_SPEED_PORT[7]), .I(counter0[7]), .T(oe_hw_test));  // in 35
IOBUF hs_expan_9(.O(sft_test_rd_data_hi[27]), .IO(HIGH_SPEED_PORT[9]), .I(counter0[9]), .T(oe_hw_test));  // in 36
IOBUF hs_expan_11(.O(sft_test_rd_data_hi[26]), .IO(HIGH_SPEED_PORT[11]), .I(counter0[11]), .T(oe_hw_test));  // in 37
IOBUF hs_expan_13(.O(sft_test_rd_data_hi[25]), .IO(HIGH_SPEED_PORT[13]), .I(counter0[13]), .T(oe_hw_test));  // in 38
IOBUF hs_expan_15(.O(sft_test_rd_data_hi[24]), .IO(HIGH_SPEED_PORT[15]), .I(counter0[15]), .T(oe_hw_test));  // in 39
IOBUF hs_expan_17(.O(sft_test_rd_data_hi[23]), .IO(HIGH_SPEED_PORT[17]), .I(counter0[17]), .T(oe_hw_test));  // in 40
IOBUF hs_expan_19(.O(sft_test_rd_data_hi[22]), .IO(HIGH_SPEED_PORT[19]), .I(counter0[19]), .T(oe_hw_test));  // in 41
IOBUF hs_expan_21(.O(sft_test_rd_data_hi[21]), .IO(HIGH_SPEED_PORT[21]), .I(counter0[21]), .T(oe_hw_test));  // in 42
IOBUF hs_expan_23(.O(sft_test_rd_data_hi[20]), .IO(HIGH_SPEED_PORT[23]), .I(counter0[23]), .T(oe_hw_test));  // in 43
IOBUF hs_expan_25(.O(sft_test_rd_data_hi[19]), .IO(HIGH_SPEED_PORT[25]), .I(counter0[25]), .T(oe_hw_test));  // in 44
IOBUF hs_expan_27(.O(sft_test_rd_data_hi[18]), .IO(HIGH_SPEED_PORT[27]), .I(counter0[27]), .T(oe_hw_test));  // in 45
IOBUF hs_expan_29(.O(sft_test_rd_data_hi[17]), .IO(HIGH_SPEED_PORT[29]), .I(counter0[29]), .T(oe_hw_test));  // in 46
IOBUF hs_expan_31(.O(sft_test_rd_data_hi[16]), .IO(HIGH_SPEED_PORT[31]), .I(counter0[31]), .T(oe_hw_test));  // in 47
IOBUF hs_expan_33(.O(sft_test_rd_data_hi[15]), .IO(HIGH_SPEED_PORT[33]), .I(counter0[33]), .T(oe_hw_test));  // in 48
IOBUF hs_expan_35(.O(sft_test_rd_data_hi[14]), .IO(HIGH_SPEED_PORT[35]), .I(counter0[35]), .T(oe_hw_test));  // in 49
IOBUF hs_expan_37(.O(sft_test_rd_data_hi[13]), .IO(HIGH_SPEED_PORT[37]), .I(counter0[37]), .T(oe_hw_test));  // in 50
IOBUF hs_expan_39(.O(sft_test_rd_data_hi[12]), .IO(HIGH_SPEED_PORT[39]), .I(counter0[39]), .T(oe_hw_test));  // in 51
IOBUF hs_expan_41(.O(sft_test_rd_data_hi[11]), .IO(HIGH_SPEED_PORT[41]), .I(counter0[41]), .T(oe_hw_test));  // in 52                 

endmodule //EXPANSION_PORT_TEST
